from cython.operator import dereference
from libc.stdlib cimport malloc

cdef extern from "bstree.h":
	cdef struct tree_node:
		int indx
		char *name
		float ratio
		void *left # to link to next struct
		void *right

	ctypedef tree_node tn

	void error()
	void insert(tn ** root, int indx, char * name, float ratio)
	void inorder(tn * root)
	int count(tn * root)
	tn * search(tn * root, int key)
	tn * delete(tn * root, int key)

cdef class BSTree:
	cdef tn *root

	def __cinit__(self):
		self.root = NULL

	cpdef void insert(self, int indx, char * name, float ratio):
		insert(&self.root, indx, name, ratio)

	cpdef void inorder(self):
		inorder(self.root)

	cpdef void remove(self, int node):
		self.root = delete(self.root, node)

	def search(self, int node):
		cdef tn *temp_tree = search(self.root, node)
		if not temp_tree:
			return "node doesn't exist"
		else:
			temp = dereference(temp_tree)
			return (temp.indx, temp.name, round(temp.ratio, 3))

	cpdef int node_count(self):
		return count(self.root)



from libc.stdlib cimport malloc
DEF INIT_SIZE = 10000000

cdef extern from "stack.h":
    ctypedef enum Type:
        INT,
        FLOAT,
        STR

    cdef union arrays:
        int * _int
        float * _float
        char _str[INIT_SIZE][100]

    cdef struct st:
        Type type
        arrays val
        int indx
        int lindx
        int size

    ctypedef st stack

    void create_stack(stack * st, int input_type)
    int is_empty(stack * st)

    # pushing functions for each type argument
    void push_int(stack * st, int new_val)
    void push_float(stack * st, float new_val)
    void push_str(stack * st, char * new_val)

    # poping functions for each type argument
    int pop_int(stack * st, int input)
    int popleft_int(stack * st, int input)
    float pop_float(stack * st, float input)
    float popleft_float(stack * st, float input)
    char * pop_str(stack * st, char * input)
    char * popleft_str(stack * st, char * input)

    void print_stack(stack * st)
    int len(stack * st)

cdef class Stack:
    cdef stack *st
    cdef int input

    def __cinit__(self, str inp):
        enums = {"int": 0, "float": 1, "str": 2}
        self.input = enums.get(inp)
        if not input:
            print("Invalid Input\nStack will be of INT type by default!")
            self.input = 0

        self.st = <stack*>malloc(sizeof(stack))

        create_stack(self.st, self.input)

    def empty(self):
        if is_empty(self.st) == 1:
            return True
        else:
            return False

    def push(self, new_val):
        if isinstance(new_val, int):
            push_int(self.st, new_val)
        elif isinstance(new_val, float):
            push_float(self.st, new_val)
        else:
            push_str(self.st, new_val.encode())

    def pop(self):
        if self.input == 0:
            return pop_int(self.st, 0)
        elif self.input == 1:
            return round(pop_float(self.st, 1.0), 2)
        else:
            return pop_str(self.st, "2")

    def popleft(self):
        if self.input == 0:
            return popleft_int(self.st, 0)
        elif self.input == 1:
            return round(popleft_float(self.st, 1.0), 2)
        else:
            return popleft_str(self.st, "2")

    cpdef void display(self):
        print_stack(self.st)

    cpdef int length(self):
        return len(self.st)



from libc.stdlib cimport malloc

cdef extern from "hash.h":
	cdef struct list_slot:
		int value
		void * next

	ctypedef list_slot ls

	cdef struct hash_table:
		int size
		ls ** slot_array
		int filled

	ctypedef hash_table ht

	ht * hash_init(int size)
	int val_search(ht * table, int value)
	int search_by_index(ht * table, int ind)
	void insert(ht * htable, int value)
	int delete_val(ht * table, int value)
	void print_hash(ht * table, int size)
	int filled_indices(ht * table)
	int len(ht * table)

cdef class HashTable:
	cdef ht * hash
	cdef int size
	cdef int filled

	def __cinit__(self, int table_size):
		self.size = table_size
		self.hash = hash_init(table_size)

	cpdef int search_value(self, int val):
		return val_search(self.hash, val)

	cpdef void insert_val(self, int val):
		insert(self.hash, val)

	cpdef int val_del(self, int val):
		return delete_val(self.hash, val)

	def display(self):
		print_hash(self.hash, self.size)

	cpdef int search_by_index(self, int ind):
		return search_by_index(self.hash, ind)
		
	cpdef int length(self):
		return len(self.hash)
		
	cpdef int occupied(self):
		return filled_indices(self.hash)


cdef extern from "pq.h":
    cdef struct linked_queue:
        char *data
        int priority
        void *next # to link to next struct

    ctypedef linked_queue pq

    void enqueue(pq **front, pq **rear, char *data, int priority)
    char *execute(pq **front, pq **rear)
    void display(pq *front, pq *rear)

cdef class PQ:
    cdef pq *front
    cdef pq *rear

    def __cinit__(self):
        self.front = NULL
        self.rear = NULL


    cpdef void push(self, char *data, int priority):
        enqueue(&self.front, &self.rear, data, priority)

    cpdef str pop(self):
        cdef char *x
        x = execute(&self.front, &self.rear)
        return x.decode()

    cpdef void display(self):
        display(self.front, self.rear)



from setuptools import setup, Extension

cython_exists = False
try:
    from Cython.Distutils import build_ext
    cython_exists = True
except ImportError:
    from distutils.command.build_ext import build_ext

if cython_exists:
    ext_modules=[
        Extension("pystack", ["stack/pystack.pyx", "stack/stack.c"]),
        Extension("pybstree", ["binary_search_tree/pybstree.pyx", "binary_search_tree/bstree.c"]),
        Extension("cpq", ["priority_queue/pqpy.pyx", "priority_queue/pq.c"]),
        Extension("pyhash", ["hash_table/pyhash.pyx", "hash_table/hash.c"])
    ]

with open("README.md", "r", encoding="utf8") as rm:
    readme = rm.read()
    
with open("requirements.txt") as rq:
    requirements = rq.read().split('\n')

setup(
      name="structpie",
      version="0.0.2",
      description="A set of python modules for data structures written in C",
      packages=["structpie"],
      install_requires=requirements,
      long_description=readme,
      include_package_data=True,
      long_description_content_type="text/markdown",
      url="https://github.com/MNoorFawi/struct-pie",
      author="Muhammad N. Fawi",
      author_email="m.noor.fawi@gmail.com",
      cmdclass = {"build_ext": build_ext},
      license="MIT",
      classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
      ext_modules = ext_modules
)